# -*- coding: utf-8 -*-
"""
Created on Tue Mar 27 17:12:35 2018

@author: Kilian
"""

# Initial settings
import os
import numpy as np
import matplotlib.pyplot as plt
import ee
import pdb

# other modules
from osgeo import gdal, ogr, osr
import pickle
import matplotlib.cm as cm
from pylab import ginput

# image processing modules
import skimage.filters as filters 
import skimage.exposure as exposure
import skimage.transform as transform
import sklearn.decomposition as decomposition
import skimage.measure as measure


# import own modules
import functions.utils as utils
import functions.sds as sds

np.seterr(all='ignore') # raise/ignore divisions by 0 and nans
plt.rcParams['axes.grid'] = True
plt.rcParams['figure.max_open_warning'] = 100
ee.Initialize()

# initial settings
cloud_thresh = 0.5      # threshold for cloud cover
plot_bool = False       # if you want the plots
prob_high = 99.9        # upper probability to clip and rescale pixel intensity
min_contour_points = 100# minimum number of points contained in each water line
output_epsg = 28356     # GDA94 / MGA Zone 56

satname = 'L8'
sitename = 'NARRA'
filepath = os.path.join(os.getcwd(), 'data', satname, sitename)
with open(os.path.join(filepath, sitename + '_timestamps' + '.pkl'), 'rb') as f:
    timestamps = pickle.load(f)
timestamps_sorted = sorted(timestamps)
with open(os.path.join(filepath, sitename + '_epsgcode' + '.pkl'), 'rb') as f:
    input_epsg = pickle.load(f)


file_path_pan = os.path.join(os.getcwd(), 'data', 'L8', 'NARRA', 'pan')
file_path_ms = os.path.join(os.getcwd(), 'data', 'L8', 'NARRA', 'ms')
file_names_pan = os.listdir(file_path_pan)
file_names_ms = os.listdir(file_path_ms)
N = len(file_names_pan)
idx_high_cloud = []
t = []
shorelines = []

for i in range(N):
    # read pan image
    fn_pan = os.path.join(file_path_pan, file_names_pan[i])
    data = gdal.Open(fn_pan, gdal.GA_ReadOnly)
    georef = np.array(data.GetGeoTransform())
    bands = [data.GetRasterBand(i + 1).ReadAsArray() for i in range(data.RasterCount)]
    im_pan = np.stack(bands, 2)[:,:,0]
    # read ms image
    fn_ms = os.path.join(file_path_ms, file_names_ms[i])
    data = gdal.Open(fn_ms, gdal.GA_ReadOnly)
    bands = [data.GetRasterBand(i + 1).ReadAsArray() for i in range(data.RasterCount)]
    im_ms = np.stack(bands, 2)
    # cloud mask
    im_qa = im_ms[:,:,5]
    cloud_mask = sds.create_cloud_mask(im_qa, satname, plot_bool)
    cloud_mask = transform.resize(cloud_mask, (im_pan.shape[0], im_pan.shape[1]),
                                order=0, preserve_range=True, 
                                mode='constant').astype('bool_')    
    # resize the image using bilinear interpolation (order 1)
    im_ms = transform.resize(im_ms,(im_pan.shape[0], im_pan.shape[1]),
                             order=1, preserve_range=True, mode='constant')
    # check if -inf or nan values and add to cloud mask
    im_inf = np.isin(im_ms[:,:,0], -np.inf)
    im_nan = np.isnan(im_ms[:,:,0])
    cloud_mask = np.logical_or(np.logical_or(cloud_mask, im_inf), im_nan)
    cloud_content = sum(sum(cloud_mask.astype(int)))/(cloud_mask.shape[0]*cloud_mask.shape[1])
    if cloud_content > cloud_thresh:
        print('skipped ' + str(i))
        idx_high_cloud.append(i)
        continue
    # rescale intensities
    im_ms = sds.rescale_image_intensity(im_ms, cloud_mask, prob_high, plot_bool)
    im_pan = sds.rescale_image_intensity(im_pan, cloud_mask, prob_high, plot_bool)
    # pansharpen rgb image
    im_ms_ps = sds.pansharpen(im_ms[:,:,[0,1,2]], im_pan, cloud_mask, plot_bool)
    # add down-sized bands for NIR and SWIR (since pansharpening is not possible)
    im_ms_ps = np.append(im_ms_ps, im_ms[:,:,[3,4]], axis=2) 
    # calculate NDWI
    im_ndwi = sds.nd_index(im_ms_ps[:,:,3], im_ms_ps[:,:,1], cloud_mask, plot_bool)
    # detect edges
    wl_pix = sds.find_wl_contours(im_ndwi, cloud_mask, min_contour_points, plot_bool)
    # convert from pixels to world coordinates
    wl_coords = sds.convert_pix2world(wl_pix, georef)
    # convert to output epsg spatial reference
    wl = sds.convert_epsg(wl_coords, input_epsg, output_epsg)
    
    plt.figure()
    cmap = cm.get_cmap('jet')

    plt.subplot(121)
    plt.imshow(im_ms_ps[:,:,[2,1,0]])
    for j,contour in enumerate(wl_pix):
        colours = cmap(np.linspace(0, 1, num=len(wl)))
        plt.plot(contour[:, 1], contour[:, 0], linewidth=2, color=colours[j,:])
    plt.axis('image')
    plt.title(file_names_pan[i])
    
    plt.subplot(122)
    centroids = []
    for j,contour in enumerate(wl):
        colours = cmap(np.linspace(0, 1, num=len(wl)))
        centroids.append([np.mean(contour[:, 0]),np.mean(contour[:, 1])])
        plt.plot(contour[:, 0], contour[:, 1], linewidth=2, color=colours[j,:])
        plt.plot(np.mean(contour[:, 0]), np.mean(contour[:, 1]), 'o', color=colours[j,:])
    plt.axis('equal')
    plt.title(file_names_pan[i])
    mng = plt.get_current_fig_manager()                                         
    mng.window.showMaximized()
    plt.tight_layout()   
    plt.draw()
    
    pt_in = np.array(ginput(1))
    
    if pt_in[0][0] < 10000:
        print('skipped m ' + str(i))
        idx_high_cloud.append(i)
        continue

    dist_centroid = [np.linalg.norm(_ - pt_in) for _ in centroids]
    shorelines.append(wl[np.argmin(dist_centroid)])
    t.append(timestamps_sorted[i])
    
#plt.figure()
#plt.axis('equal')
#for j in range(len(shorelines)):
#    plt.plot(shorelines[j][:,0], shorelines[j][:,1])
#plt.draw()
    
output = {'t':t, 'shorelines':shorelines}

with open(os.path.join(filepath, sitename + '_output' + '.pkl'), 'wb') as f:
    pickle.dump(output, f)
    

